// CaptureTest3.cpp
// This file is public domain software (PDS).
// Copyright (C) 2017 Katayama Hirofumi MZ <katayama.hirofumi.mz@gmail.com>
#include "MWindowBase.hpp"
#include <cstdio>

class CaptureTest3ChildWindow : public MWindowBase
{
public:
    BOOL m_dragging;
    HWND m_hwndParent;

    CaptureTest3ChildWindow()
    {
        m_dragging = FALSE;
        m_hwndParent = NULL;
    }

    virtual LRESULT CALLBACK
    WindowProcDx(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
    {
        switch (uMsg)
        {
            HANDLE_MSG(hwnd, WM_LBUTTONDOWN, OnLButtonDown);
            HANDLE_MSG(hwnd, WM_LBUTTONUP, OnLButtonUp);
            HANDLE_MSG(hwnd, WM_MOUSEMOVE, OnMouseMove);
            HANDLE_MSG(hwnd, WM_PAINT, OnPaint);
            case WM_CANCELMODE:
                std::printf("m_child: WM_CANCELMODE\n");
                break;
            case WM_CAPTURECHANGED:
                std::printf("m_child: WM_CAPTURECHANGED\n");
                break;
        }
        return DefaultProcDx();
    }

    void OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
    {
        m_dragging = TRUE;
        SetCapture(hwnd);
    }

    void OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags)
    {
        InvalidateRect(hwnd, NULL, TRUE);
    }

    void OnLButtonUp(HWND hwnd, int x, int y, UINT keyFlags)
    {
        SendMessage(m_hwndParent, WM_COMMAND, 999, 0);
        ReleaseCapture();
        m_dragging = FALSE;
    }

    void OnPaint(HWND hwnd)
    {
        PAINTSTRUCT ps;
        HDC hDC = BeginPaint(hwnd, &ps);
        if (hDC)
        {
            RECT rc;
            GetClientRect(hwnd, &rc);
            DrawText(hDC, TEXT("Drag Me"), -1, &rc, DT_SINGLELINE | DT_CENTER | DT_VCENTER);

            if (m_dragging)
            {
                POINT pt;
                GetCursorPos(&pt);
                ScreenToClient(hwnd, &pt);
                Ellipse(hDC, pt.x - 5, pt.y - 5, pt.x + 5, pt.y + 5);
            }

            EndPaint(hwnd, &ps);
        }
    }
};

class CaptureTest3ChildWindow2 : public MWindowBase
{
public:
    HWND m_hwndParent;

    CaptureTest3ChildWindow2() : m_hwndParent(NULL)
    {
    }

    virtual LPCTSTR GetWndClassNameDx() const
    {
        return TEXT("CaptureTest3 another child window");
    }

    virtual void ModifyWndClassDx(WNDCLASSEX& wcx)
    {
        wcx.hbrBackground = CreateSolidBrush(RGB(255, 255, 0)); // yellow
    }

    virtual LRESULT CALLBACK
    WindowProcDx(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
    {
        switch (uMsg)
        {
            HANDLE_MSG(hwnd, WM_CREATE, OnCreate);
            HANDLE_MSG(hwnd, WM_LBUTTONUP, OnLButtonUp);
            HANDLE_MSG(hwnd, WM_DESTROY, OnDestroy);
            case WM_CANCELMODE:
                std::printf("m_child2: WM_CANCELMODE\n");
                break;
            case WM_CAPTURECHANGED:
                std::printf("m_child2: WM_CAPTURECHANGED\n");
                break;
        }
        return DefaultProcDx();
    }

    BOOL OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct)
    {
        EnableWindow(m_hwndParent, FALSE);
        return TRUE;
    }

    void OnLButtonUp(HWND hwnd, int x, int y, UINT keyFlags)
    {
        DestroyWindow(hwnd);
    }

    void OnDestroy(HWND hwnd)
    {
        EnableWindow(m_hwndParent, TRUE);
    }
};

class CaptureTest3 : public MWindowBase
{
    CaptureTest3ChildWindow     m_child;
    CaptureTest3ChildWindow2    m_child2;
public:
    CaptureTest3()
    {
    }

    virtual LRESULT CALLBACK
    WindowProcDx(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
    {
        switch (uMsg)
        {
            HANDLE_MSG(hwnd, WM_CREATE, OnCreate);
            HANDLE_MSG(hwnd, WM_DESTROY, OnDestroy);
            HANDLE_MSG(hwnd, WM_SIZE, OnSize);
            HANDLE_MSG(hwnd, WM_COMMAND, OnCommand);
            HANDLE_MSG(hwnd, WM_TIMER, OnTimer);
            case WM_CANCELMODE:
                std::printf("MainWnd: WM_CANCELMODE\n");
                break;
            case WM_CAPTURECHANGED:
                std::printf("MainWnd: WM_CAPTURECHANGED\n");
                break;
        }
        return DefaultProcDx();
    }

    BOOL OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct)
    {
        m_child.CreateWindowDx(hwnd, NULL, WS_CHILD | WS_VISIBLE);
        m_child.m_hwndParent = hwnd;
        SetTimer(hwnd, 666, 1000, NULL);
        return TRUE;
    }

    void OnTimer(HWND hwnd, UINT id)
    {
        if (id == 666)
        {
            std::printf("GetCapture(): %p\n", (void *)GetCapture());
        }
    }

    void OnSize(HWND hwnd, UINT state, int cx, int cy)
    {
        MoveWindow(m_child, 0, 0, cx, cy, TRUE);
    }

    void OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
    {
        if (id == 999)
        {
            if (m_child2)
                DestroyWindow(m_child2);

            m_child2.m_hwndParent = hwnd;
            m_child2.CreateWindowDx(hwnd, TEXT("another window"),
                WS_POPUPWINDOW, 0, CW_USEDEFAULT, CW_USEDEFAULT, 80, 80);
            m_child2.CenterWindowDx();

            ShowWindow(m_child2, SW_SHOWNOACTIVATE);
            UpdateWindow(m_child2);
        }
    }

    void OnDestroy(HWND hwnd)
    {
        KillTimer(hwnd, 666);
        PostQuitMessage(0);
    }

    BOOL StartDx(LPTSTR lpCmdLine, INT nCmdShow)
    {
        if (!CreateWindowDx(NULL, TEXT("CaptureTest3"), WS_OVERLAPPEDWINDOW, 0,
                            CW_USEDEFAULT, CW_USEDEFAULT, 350, 220))
        {
            MsgBoxDx(TEXT("CreateWindowDx failed"), NULL, MB_ICONERROR);
            return FALSE;
        }

        ShowWindow(m_hwnd, nCmdShow);
        UpdateWindow(m_hwnd);
        return TRUE;
    }

    INT_PTR RunDx()
    {
        MSG msg;
        while (GetMessage(&msg, NULL, 0, 0))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        return INT(msg.wParam);
    }
};

int main(void)
{
    CaptureTest3 app;
    if (app.StartDx(TEXT(""), SW_SHOWNORMAL))
    {
        return app.RunDx();
    }
    return -1;
}
